@@ -4,7 +4,7 @@ package Convert::EDS::XDD;
 no warnings 'uninitialized';
 
 # ABSTRACT: Converts the CANopen EDS format to XDD
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 use Carp;
 use Config::Tiny;
@@ -170,7 +170,9 @@ sub eds2xdd_string {
     $writer->startTag("ObjectList");
 
     my ($in_sublist, $in_6000, $in_2000, $in_1000) = (0) x 4;
-    foreach my $section (sort keys %{$eds}) {
+    my @sections = (sort keys %{$eds});
+    foreach my $section_index (0 .. @sections - 1) {
+        my $section = $sections[$section_index];
         unless ($section =~ /([[:xdigit:]]{4})(?:sub([[:xdigit:]]))|([[:xdigit:]]{4})/) {
             carp "Ignoring unknown section $section\n";
             next;
@@ -197,11 +199,11 @@ sub eds2xdd_string {
             $in_sublist = 0;
 
             unshift @object, index => sprintf('%04X', hex($index));
-            if (not $obj->{SubNumber}) {
-                $writer->emptyTag("Object", @object);
-            } else {
+            if ($sections[$section_index+1] =~ /^${index}sub/) {
                 $writer->startTag("Object", @object);
                 $in_sublist = 1;
+            } else {
+                $writer->emptyTag("Object", @object);
             }
         } else {
             unshift @object, subIndex => sprintf('%02X', hex($subindex));
@@ -209,7 +211,9 @@ sub eds2xdd_string {
         }
     }
 
+    $writer->endTag("Object") if $in_sublist;
     $writer->endTag("ObjectList");
+
     my $ObjectList = $writer->end();
     my $xdd = $template;
     if ($xdd =~ s/^([ \t]+?)\$ObjectList/\$ObjectList/m) {
